# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))

import sphinx_rtd_theme

import fsrl

# Get the version string
version = fsrl.__version__

# -- Project information -----------------------------------------------------

project = "FSRL"
copyright = "2023, FSRL contributors"
author = "FSRL contributors"

# The full version, including alpha/beta/rc tags
release = version

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.doctest",
    "sphinx.ext.intersphinx",
    "sphinx.ext.coverage",
    # 'sphinx.ext.imgmath',
    "sphinx.ext.mathjax",
    "sphinx.ext.ifconfig",
    "sphinx.ext.viewcode",
    "sphinx.ext.githubpages",
    "sphinxcontrib.bibtex"
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]
source_suffix = [".rst"]
master_doc = "index"

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store", "misc"]
# autodoc_default_options = {
#     "special-members":
#     ", ".join([
#         "__len__",
#         "__call__",
#         "__getitem__",
#         "__setitem__",
#         # "__getattr__",
#         # "__setattr__",
#     ])
# }
autodoc_member_order = "bysource"
bibtex_bibfiles = ['refs.bib']
bibtex_reference_style = 'author_year'

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "sphinx_rtd_theme"
html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

html_logo = "_static/images/fsrl-logo.png"


def setup(app):
    app.add_js_file("https://cdn.jsdelivr.net/npm/vega@5.20.2")
    app.add_js_file("https://cdn.jsdelivr.net/npm/vega-lite@5.1.0")
    app.add_js_file("https://cdn.jsdelivr.net/npm/vega-embed@6.17.0")
    app.add_js_file("https://code.jquery.com/jquery-3.6.0.min.js")

    # app.add_js_file("js/copybutton.js")
    app.add_js_file("js/benchmark.js")
    app.add_css_file("css/style.css")


# -- Extension configuration -------------------------------------------------

# -- Options for intersphinx extension ---------------------------------------

# Example configuration for intersphinx: refer to the Python standard library.
# intersphinx_mapping = {'https://docs.python.org/3/': None}

# -- Options for todo extension ----------------------------------------------

# If true, `todo` and `todoList` produce output, else they produce nothing.
# todo_include_todos = False
